@@ -29,9 +29,12 @@ except ImportError:
 try:
     import locale
     locale.setlocale(locale.LC_CTYPE, "")
-except (ImportError, locale.error):
+except (ImportError, locale.Error):
     pass
 
+# Encoding for file names
+filesystemencoding = sys.getfilesystemencoding()
+
 encoding = "ascii"
 if sys.platform == 'win32':
     # On Windows, we could use "mbcs". However, to give the user
@@ -48,7 +51,7 @@ else:
         # resulting codeset may be unknown to Python. We ignore all
         # these problems, falling back to ASCII
         encoding = locale.nl_langinfo(locale.CODESET)
-        if encoding is None:
+        if encoding is None or encoding is '':
             # situation occurs on Mac OS X
             encoding = 'ascii'
         codecs.lookup(encoding)
@@ -58,7 +61,7 @@ else:
         # bugs that can cause ValueError.
         try:
             encoding = locale.getdefaultlocale()[1]
-            if encoding is None:
+            if encoding is None or encoding is '':
                 # situation occurs on Mac OS X
                 encoding = 'ascii'
             codecs.lookup(encoding)
@@ -374,6 +377,7 @@ class IOBinding:
         try:
             f = open(filename, "wb")
             f.write(chars)
+            f.flush()
             f.close()
             return True
         except IOError, msg:
@@ -517,7 +521,10 @@ class IOBinding:
         if not self.opendialog:
             self.opendialog = tkFileDialog.Open(master=self.text,
                                                 filetypes=self.filetypes)
-        return self.opendialog.show(initialdir=dir, initialfile=base)
+        filename = self.opendialog.show(initialdir=dir, initialfile=base)
+        if isinstance(filename, unicode):
+            filename = filename.encode(filesystemencoding)
+        return filename
 
     def defaultfilename(self, mode="open"):
         if self.filename:
@@ -536,11 +543,14 @@ class IOBinding:
         if not self.savedialog:
             self.savedialog = tkFileDialog.SaveAs(master=self.text,
                                                   filetypes=self.filetypes)
-        return self.savedialog.show(initialdir=dir, initialfile=base)
+        filename = self.savedialog.show(initialdir=dir, initialfile=base)
+        if isinstance(filename, unicode):
+            filename = filename.encode(filesystemencoding)
+        return filename
 
     def updaterecentfileslist(self,filename):
         "Update recent file list on all editor windows"
-        self.editwin.UpdateRecentFilesList(filename)
+        self.editwin.update_recent_files_list(filename)
 
 def test():
     root = Tk()
