@@ -5,26 +5,17 @@
 
 
 
-#ifdef _WIN32
-#include "pywintoolbox.h"
-#else
-#include "macglue.h"
 #include "pymactoolbox.h"
-#endif
 
 /* Macro to test whether a weak-loaded CFM function exists */
 #define PyMac_PRECHECK(rtn) do { if ( &rtn == NULL )  {\
-    	PyErr_SetString(PyExc_NotImplementedError, \
-    	"Not available in this shared library/OS version"); \
-    	return NULL; \
+        PyErr_SetString(PyExc_NotImplementedError, \
+        "Not available in this shared library/OS version"); \
+        return NULL; \
     }} while(0)
 
 
-#ifdef WITHOUT_FRAMEWORKS
-#include <Icons.h>
-#else
 #include <Carbon/Carbon.h>
-#endif
 
 
 static PyObject *Icn_Error;
@@ -606,23 +597,6 @@ static PyObject *Icn_PlotCIconHandle(PyObject *_self, PyObject *_args)
 	return _res;
 }
 
-#if !TARGET_API_MAC_CARBON
-
-static PyObject *Icn_IconServicesTerminate(PyObject *_self, PyObject *_args)
-{
-	PyObject *_res = NULL;
-#ifndef IconServicesTerminate
-	PyMac_PRECHECK(IconServicesTerminate);
-#endif
-	if (!PyArg_ParseTuple(_args, ""))
-		return NULL;
-	IconServicesTerminate();
-	Py_INCREF(Py_None);
-	_res = Py_None;
-	return _res;
-}
-#endif
-
 static PyObject *Icn_IconRefToIconFamily(PyObject *_self, PyObject *_args)
 {
 	PyObject *_res = NULL;
@@ -932,6 +906,32 @@ static PyObject *Icn_RegisterIconRefFromResource(PyObject *_self, PyObject *_arg
 	return _res;
 }
 
+static PyObject *Icn_RegisterIconRefFromFSRef(PyObject *_self, PyObject *_args)
+{
+	PyObject *_res = NULL;
+	OSStatus _err;
+	OSType creator;
+	OSType iconType;
+	FSRef iconFile;
+	IconRef theIconRef;
+#ifndef RegisterIconRefFromFSRef
+	PyMac_PRECHECK(RegisterIconRefFromFSRef);
+#endif
+	if (!PyArg_ParseTuple(_args, "O&O&O&",
+	                      PyMac_GetOSType, &creator,
+	                      PyMac_GetOSType, &iconType,
+	                      PyMac_GetFSRef, &iconFile))
+		return NULL;
+	_err = RegisterIconRefFromFSRef(creator,
+	                                iconType,
+	                                &iconFile,
+	                                &theIconRef);
+	if (_err != noErr) return PyMac_Error(_err);
+	_res = Py_BuildValue("O&",
+	                     ResObj_New, theIconRef);
+	return _res;
+}
+
 static PyObject *Icn_UnregisterIconRef(PyObject *_self, PyObject *_args)
 {
 	PyObject *_res = NULL;
@@ -1338,8 +1338,6 @@ static PyObject *Icn_IsIconRefMaskEmpty(PyObject *_self, PyObject *_args)
 	return _res;
 }
 
-#if TARGET_API_MAC_CARBON
-
 static PyObject *Icn_GetIconRefVariant(PyObject *_self, PyObject *_args)
 {
 	PyObject *_res = NULL;
@@ -1362,9 +1360,6 @@ static PyObject *Icn_GetIconRefVariant(PyObject *_self, PyObject *_args)
 	                     outTransform);
 	return _res;
 }
-#endif
-
-#if TARGET_API_MAC_CARBON
 
 static PyObject *Icn_RegisterIconRefFromIconFile(PyObject *_self, PyObject *_args)
 {
@@ -1391,9 +1386,6 @@ static PyObject *Icn_RegisterIconRefFromIconFile(PyObject *_self, PyObject *_arg
 	                     ResObj_New, theIconRef);
 	return _res;
 }
-#endif
-
-#if TARGET_API_MAC_CARBON
 
 static PyObject *Icn_ReadIconFile(PyObject *_self, PyObject *_args)
 {
@@ -1414,9 +1406,26 @@ static PyObject *Icn_ReadIconFile(PyObject *_self, PyObject *_args)
 	                     ResObj_New, iconFamily);
 	return _res;
 }
-#endif
 
-#if TARGET_API_MAC_CARBON
+static PyObject *Icn_ReadIconFromFSRef(PyObject *_self, PyObject *_args)
+{
+	PyObject *_res = NULL;
+	OSStatus _err;
+	FSRef ref;
+	IconFamilyHandle iconFamily;
+#ifndef ReadIconFromFSRef
+	PyMac_PRECHECK(ReadIconFromFSRef);
+#endif
+	if (!PyArg_ParseTuple(_args, "O&",
+	                      PyMac_GetFSRef, &ref))
+		return NULL;
+	_err = ReadIconFromFSRef(&ref,
+	                         &iconFamily);
+	if (_err != noErr) return PyMac_Error(_err);
+	_res = Py_BuildValue("O&",
+	                     ResObj_New, iconFamily);
+	return _res;
+}
 
 static PyObject *Icn_WriteIconFile(PyObject *_self, PyObject *_args)
 {
@@ -1438,146 +1447,132 @@ static PyObject *Icn_WriteIconFile(PyObject *_self, PyObject *_args)
 	_res = Py_None;
 	return _res;
 }
-#endif
 
 static PyMethodDef Icn_methods[] = {
 	{"GetCIcon", (PyCFunction)Icn_GetCIcon, 1,
-	 "(SInt16 iconID) -> (CIconHandle _rv)"},
+	 PyDoc_STR("(SInt16 iconID) -> (CIconHandle _rv)")},
 	{"PlotCIcon", (PyCFunction)Icn_PlotCIcon, 1,
-	 "(Rect theRect, CIconHandle theIcon) -> None"},
+	 PyDoc_STR("(Rect theRect, CIconHandle theIcon) -> None")},
 	{"DisposeCIcon", (PyCFunction)Icn_DisposeCIcon, 1,
-	 "(CIconHandle theIcon) -> None"},
+	 PyDoc_STR("(CIconHandle theIcon) -> None")},
 	{"GetIcon", (PyCFunction)Icn_GetIcon, 1,
-	 "(SInt16 iconID) -> (Handle _rv)"},
+	 PyDoc_STR("(SInt16 iconID) -> (Handle _rv)")},
 	{"PlotIcon", (PyCFunction)Icn_PlotIcon, 1,
-	 "(Rect theRect, Handle theIcon) -> None"},
+	 PyDoc_STR("(Rect theRect, Handle theIcon) -> None")},
 	{"PlotIconID", (PyCFunction)Icn_PlotIconID, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, SInt16 theResID) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, SInt16 theResID) -> None")},
 	{"NewIconSuite", (PyCFunction)Icn_NewIconSuite, 1,
-	 "() -> (IconSuiteRef theIconSuite)"},
+	 PyDoc_STR("() -> (IconSuiteRef theIconSuite)")},
 	{"AddIconToSuite", (PyCFunction)Icn_AddIconToSuite, 1,
-	 "(Handle theIconData, IconSuiteRef theSuite, ResType theType) -> None"},
+	 PyDoc_STR("(Handle theIconData, IconSuiteRef theSuite, ResType theType) -> None")},
 	{"GetIconFromSuite", (PyCFunction)Icn_GetIconFromSuite, 1,
-	 "(IconSuiteRef theSuite, ResType theType) -> (Handle theIconData)"},
+	 PyDoc_STR("(IconSuiteRef theSuite, ResType theType) -> (Handle theIconData)")},
 	{"GetIconSuite", (PyCFunction)Icn_GetIconSuite, 1,
-	 "(SInt16 theResID, IconSelectorValue selector) -> (IconSuiteRef theIconSuite)"},
+	 PyDoc_STR("(SInt16 theResID, IconSelectorValue selector) -> (IconSuiteRef theIconSuite)")},
 	{"DisposeIconSuite", (PyCFunction)Icn_DisposeIconSuite, 1,
-	 "(IconSuiteRef theIconSuite, Boolean disposeData) -> None"},
+	 PyDoc_STR("(IconSuiteRef theIconSuite, Boolean disposeData) -> None")},
 	{"PlotIconSuite", (PyCFunction)Icn_PlotIconSuite, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, IconSuiteRef theIconSuite) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, IconSuiteRef theIconSuite) -> None")},
 	{"LoadIconCache", (PyCFunction)Icn_LoadIconCache, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, IconCacheRef theIconCache) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, IconCacheRef theIconCache) -> None")},
 	{"GetLabel", (PyCFunction)Icn_GetLabel, 1,
-	 "(SInt16 labelNumber, Str255 labelString) -> (RGBColor labelColor)"},
+	 PyDoc_STR("(SInt16 labelNumber, Str255 labelString) -> (RGBColor labelColor)")},
 	{"PtInIconID", (PyCFunction)Icn_PtInIconID, 1,
-	 "(Point testPt, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> (Boolean _rv)"},
+	 PyDoc_STR("(Point testPt, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> (Boolean _rv)")},
 	{"PtInIconSuite", (PyCFunction)Icn_PtInIconSuite, 1,
-	 "(Point testPt, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> (Boolean _rv)"},
+	 PyDoc_STR("(Point testPt, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> (Boolean _rv)")},
 	{"RectInIconID", (PyCFunction)Icn_RectInIconID, 1,
-	 "(Rect testRect, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> (Boolean _rv)"},
+	 PyDoc_STR("(Rect testRect, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> (Boolean _rv)")},
 	{"RectInIconSuite", (PyCFunction)Icn_RectInIconSuite, 1,
-	 "(Rect testRect, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> (Boolean _rv)"},
+	 PyDoc_STR("(Rect testRect, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> (Boolean _rv)")},
 	{"IconIDToRgn", (PyCFunction)Icn_IconIDToRgn, 1,
-	 "(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> None"},
+	 PyDoc_STR("(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, SInt16 iconID) -> None")},
 	{"IconSuiteToRgn", (PyCFunction)Icn_IconSuiteToRgn, 1,
-	 "(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> None"},
+	 PyDoc_STR("(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, IconSuiteRef theIconSuite) -> None")},
 	{"SetSuiteLabel", (PyCFunction)Icn_SetSuiteLabel, 1,
-	 "(IconSuiteRef theSuite, SInt16 theLabel) -> None"},
+	 PyDoc_STR("(IconSuiteRef theSuite, SInt16 theLabel) -> None")},
 	{"GetSuiteLabel", (PyCFunction)Icn_GetSuiteLabel, 1,
-	 "(IconSuiteRef theSuite) -> (SInt16 _rv)"},
+	 PyDoc_STR("(IconSuiteRef theSuite) -> (SInt16 _rv)")},
 	{"PlotIconHandle", (PyCFunction)Icn_PlotIconHandle, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, Handle theIcon) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, Handle theIcon) -> None")},
 	{"PlotSICNHandle", (PyCFunction)Icn_PlotSICNHandle, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, Handle theSICN) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, Handle theSICN) -> None")},
 	{"PlotCIconHandle", (PyCFunction)Icn_PlotCIconHandle, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, CIconHandle theCIcon) -> None"},
-
-#if !TARGET_API_MAC_CARBON
-	{"IconServicesTerminate", (PyCFunction)Icn_IconServicesTerminate, 1,
-	 "() -> None"},
-#endif
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, CIconHandle theCIcon) -> None")},
 	{"IconRefToIconFamily", (PyCFunction)Icn_IconRefToIconFamily, 1,
-	 "(IconRef theIconRef, IconSelectorValue whichIcons) -> (IconFamilyHandle iconFamily)"},
+	 PyDoc_STR("(IconRef theIconRef, IconSelectorValue whichIcons) -> (IconFamilyHandle iconFamily)")},
 	{"IconFamilyToIconSuite", (PyCFunction)Icn_IconFamilyToIconSuite, 1,
-	 "(IconFamilyHandle iconFamily, IconSelectorValue whichIcons) -> (IconSuiteRef iconSuite)"},
+	 PyDoc_STR("(IconFamilyHandle iconFamily, IconSelectorValue whichIcons) -> (IconSuiteRef iconSuite)")},
 	{"IconSuiteToIconFamily", (PyCFunction)Icn_IconSuiteToIconFamily, 1,
-	 "(IconSuiteRef iconSuite, IconSelectorValue whichIcons) -> (IconFamilyHandle iconFamily)"},
+	 PyDoc_STR("(IconSuiteRef iconSuite, IconSelectorValue whichIcons) -> (IconFamilyHandle iconFamily)")},
 	{"SetIconFamilyData", (PyCFunction)Icn_SetIconFamilyData, 1,
-	 "(IconFamilyHandle iconFamily, OSType iconType, Handle h) -> None"},
+	 PyDoc_STR("(IconFamilyHandle iconFamily, OSType iconType, Handle h) -> None")},
 	{"GetIconFamilyData", (PyCFunction)Icn_GetIconFamilyData, 1,
-	 "(IconFamilyHandle iconFamily, OSType iconType, Handle h) -> None"},
+	 PyDoc_STR("(IconFamilyHandle iconFamily, OSType iconType, Handle h) -> None")},
 	{"GetIconRefOwners", (PyCFunction)Icn_GetIconRefOwners, 1,
-	 "(IconRef theIconRef) -> (UInt16 owners)"},
+	 PyDoc_STR("(IconRef theIconRef) -> (UInt16 owners)")},
 	{"AcquireIconRef", (PyCFunction)Icn_AcquireIconRef, 1,
-	 "(IconRef theIconRef) -> None"},
+	 PyDoc_STR("(IconRef theIconRef) -> None")},
 	{"ReleaseIconRef", (PyCFunction)Icn_ReleaseIconRef, 1,
-	 "(IconRef theIconRef) -> None"},
+	 PyDoc_STR("(IconRef theIconRef) -> None")},
 	{"GetIconRefFromFile", (PyCFunction)Icn_GetIconRefFromFile, 1,
-	 "(FSSpec theFile) -> (IconRef theIconRef, SInt16 theLabel)"},
+	 PyDoc_STR("(FSSpec theFile) -> (IconRef theIconRef, SInt16 theLabel)")},
 	{"GetIconRef", (PyCFunction)Icn_GetIconRef, 1,
-	 "(SInt16 vRefNum, OSType creator, OSType iconType) -> (IconRef theIconRef)"},
+	 PyDoc_STR("(SInt16 vRefNum, OSType creator, OSType iconType) -> (IconRef theIconRef)")},
 	{"GetIconRefFromFolder", (PyCFunction)Icn_GetIconRefFromFolder, 1,
-	 "(SInt16 vRefNum, SInt32 parentFolderID, SInt32 folderID, SInt8 attributes, SInt8 accessPrivileges) -> (IconRef theIconRef)"},
+	 PyDoc_STR("(SInt16 vRefNum, SInt32 parentFolderID, SInt32 folderID, SInt8 attributes, SInt8 accessPrivileges) -> (IconRef theIconRef)")},
 	{"RegisterIconRefFromIconFamily", (PyCFunction)Icn_RegisterIconRefFromIconFamily, 1,
-	 "(OSType creator, OSType iconType, IconFamilyHandle iconFamily) -> (IconRef theIconRef)"},
+	 PyDoc_STR("(OSType creator, OSType iconType, IconFamilyHandle iconFamily) -> (IconRef theIconRef)")},
 	{"RegisterIconRefFromResource", (PyCFunction)Icn_RegisterIconRefFromResource, 1,
-	 "(OSType creator, OSType iconType, FSSpec resourceFile, SInt16 resourceID) -> (IconRef theIconRef)"},
+	 PyDoc_STR("(OSType creator, OSType iconType, FSSpec resourceFile, SInt16 resourceID) -> (IconRef theIconRef)")},
+	{"RegisterIconRefFromFSRef", (PyCFunction)Icn_RegisterIconRefFromFSRef, 1,
+	 PyDoc_STR("(OSType creator, OSType iconType, FSRef iconFile) -> (IconRef theIconRef)")},
 	{"UnregisterIconRef", (PyCFunction)Icn_UnregisterIconRef, 1,
-	 "(OSType creator, OSType iconType) -> None"},
+	 PyDoc_STR("(OSType creator, OSType iconType) -> None")},
 	{"UpdateIconRef", (PyCFunction)Icn_UpdateIconRef, 1,
-	 "(IconRef theIconRef) -> None"},
+	 PyDoc_STR("(IconRef theIconRef) -> None")},
 	{"OverrideIconRefFromResource", (PyCFunction)Icn_OverrideIconRefFromResource, 1,
-	 "(IconRef theIconRef, FSSpec resourceFile, SInt16 resourceID) -> None"},
+	 PyDoc_STR("(IconRef theIconRef, FSSpec resourceFile, SInt16 resourceID) -> None")},
 	{"OverrideIconRef", (PyCFunction)Icn_OverrideIconRef, 1,
-	 "(IconRef oldIconRef, IconRef newIconRef) -> None"},
+	 PyDoc_STR("(IconRef oldIconRef, IconRef newIconRef) -> None")},
 	{"RemoveIconRefOverride", (PyCFunction)Icn_RemoveIconRefOverride, 1,
-	 "(IconRef theIconRef) -> None"},
+	 PyDoc_STR("(IconRef theIconRef) -> None")},
 	{"CompositeIconRef", (PyCFunction)Icn_CompositeIconRef, 1,
-	 "(IconRef backgroundIconRef, IconRef foregroundIconRef) -> (IconRef compositeIconRef)"},
+	 PyDoc_STR("(IconRef backgroundIconRef, IconRef foregroundIconRef) -> (IconRef compositeIconRef)")},
 	{"IsIconRefComposite", (PyCFunction)Icn_IsIconRefComposite, 1,
-	 "(IconRef compositeIconRef) -> (IconRef backgroundIconRef, IconRef foregroundIconRef)"},
+	 PyDoc_STR("(IconRef compositeIconRef) -> (IconRef backgroundIconRef, IconRef foregroundIconRef)")},
 	{"IsValidIconRef", (PyCFunction)Icn_IsValidIconRef, 1,
-	 "(IconRef theIconRef) -> (Boolean _rv)"},
+	 PyDoc_STR("(IconRef theIconRef) -> (Boolean _rv)")},
 	{"PlotIconRef", (PyCFunction)Icn_PlotIconRef, 1,
-	 "(Rect theRect, IconAlignmentType align, IconTransformType transform, IconServicesUsageFlags theIconServicesUsageFlags, IconRef theIconRef) -> None"},
+	 PyDoc_STR("(Rect theRect, IconAlignmentType align, IconTransformType transform, IconServicesUsageFlags theIconServicesUsageFlags, IconRef theIconRef) -> None")},
 	{"PtInIconRef", (PyCFunction)Icn_PtInIconRef, 1,
-	 "(Point testPt, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags theIconServicesUsageFlags, IconRef theIconRef) -> (Boolean _rv)"},
+	 PyDoc_STR("(Point testPt, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags theIconServicesUsageFlags, IconRef theIconRef) -> (Boolean _rv)")},
 	{"RectInIconRef", (PyCFunction)Icn_RectInIconRef, 1,
-	 "(Rect testRect, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> (Boolean _rv)"},
+	 PyDoc_STR("(Rect testRect, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> (Boolean _rv)")},
 	{"IconRefToRgn", (PyCFunction)Icn_IconRefToRgn, 1,
-	 "(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> None"},
+	 PyDoc_STR("(RgnHandle theRgn, Rect iconRect, IconAlignmentType align, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> None")},
 	{"GetIconSizesFromIconRef", (PyCFunction)Icn_GetIconSizesFromIconRef, 1,
-	 "(IconSelectorValue iconSelectorInput, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> (IconSelectorValue iconSelectorOutputPtr)"},
+	 PyDoc_STR("(IconSelectorValue iconSelectorInput, IconServicesUsageFlags iconServicesUsageFlags, IconRef theIconRef) -> (IconSelectorValue iconSelectorOutputPtr)")},
 	{"FlushIconRefs", (PyCFunction)Icn_FlushIconRefs, 1,
-	 "(OSType creator, OSType iconType) -> None"},
+	 PyDoc_STR("(OSType creator, OSType iconType) -> None")},
 	{"FlushIconRefsByVolume", (PyCFunction)Icn_FlushIconRefsByVolume, 1,
-	 "(SInt16 vRefNum) -> None"},
+	 PyDoc_STR("(SInt16 vRefNum) -> None")},
 	{"SetCustomIconsEnabled", (PyCFunction)Icn_SetCustomIconsEnabled, 1,
-	 "(SInt16 vRefNum, Boolean enableCustomIcons) -> None"},
+	 PyDoc_STR("(SInt16 vRefNum, Boolean enableCustomIcons) -> None")},
 	{"GetCustomIconsEnabled", (PyCFunction)Icn_GetCustomIconsEnabled, 1,
-	 "(SInt16 vRefNum) -> (Boolean customIconsEnabled)"},
+	 PyDoc_STR("(SInt16 vRefNum) -> (Boolean customIconsEnabled)")},
 	{"IsIconRefMaskEmpty", (PyCFunction)Icn_IsIconRefMaskEmpty, 1,
-	 "(IconRef iconRef) -> (Boolean _rv)"},
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("(IconRef iconRef) -> (Boolean _rv)")},
 	{"GetIconRefVariant", (PyCFunction)Icn_GetIconRefVariant, 1,
-	 "(IconRef inIconRef, OSType inVariant) -> (IconRef _rv, IconTransformType outTransform)"},
-#endif
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("(IconRef inIconRef, OSType inVariant) -> (IconRef _rv, IconTransformType outTransform)")},
 	{"RegisterIconRefFromIconFile", (PyCFunction)Icn_RegisterIconRefFromIconFile, 1,
-	 "(OSType creator, OSType iconType, FSSpec iconFile) -> (IconRef theIconRef)"},
-#endif
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("(OSType creator, OSType iconType, FSSpec iconFile) -> (IconRef theIconRef)")},
 	{"ReadIconFile", (PyCFunction)Icn_ReadIconFile, 1,
-	 "(FSSpec iconFile) -> (IconFamilyHandle iconFamily)"},
-#endif
-
-#if TARGET_API_MAC_CARBON
+	 PyDoc_STR("(FSSpec iconFile) -> (IconFamilyHandle iconFamily)")},
+	{"ReadIconFromFSRef", (PyCFunction)Icn_ReadIconFromFSRef, 1,
+	 PyDoc_STR("(FSRef ref) -> (IconFamilyHandle iconFamily)")},
 	{"WriteIconFile", (PyCFunction)Icn_WriteIconFile, 1,
-	 "(IconFamilyHandle iconFamily, FSSpec iconFile) -> None"},
-#endif
+	 PyDoc_STR("(IconFamilyHandle iconFamily, FSSpec iconFile) -> None")},
 	{NULL, NULL, 0}
 };
 
