@@ -45,17 +45,38 @@ DIR=`dirname $DIR`
 ## Configurable options
 
 FAILURE_SUBJECT="Python Regression Test Failures"
+#FAILURE_MAILTO="YOUR_ACCOUNT@gmail.com"
 FAILURE_MAILTO="python-checkins@python.org"
-FAILURE_CC="nnorwitz@gmail.com"
+#FAILURE_CC="optional--uncomment and set to desired address"
 
 REMOTE_SYSTEM="neal@dinsdale.python.org"
-REMOTE_DIR="/data/ftp.python.org/pub/docs.python.org/dev/2.4"
+REMOTE_DIR="/data/ftp.python.org/pub/docs.python.org/dev/2.5"
 RESULT_FILE="$DIR/build/index.html"
-INSTALL_DIR="/tmp/python-test-2.4/local"
+INSTALL_DIR="/tmp/python-test-2.5/local"
 RSYNC_OPTS="-aC -e ssh"
 
+# Always run the installed version of Python.
+PYTHON=$INSTALL_DIR/bin/python
+
+# Python options and regression test program that should always be run.
+REGRTEST_ARGS="-E -tt $INSTALL_DIR/lib/python2.5/test/regrtest.py"
+
 REFLOG="build/reflog.txt.out"
-# Change this flag to "yes" for old releases to just update/build the docs.
+# These tests are not stable and falsely report leaks sometimes.
+# The entire leak report will be mailed if any test not in this list leaks.
+# Note: test_XXX (none currently) really leak, but are disabled
+# so we don't send spam.  Any test which really leaks should only 
+# be listed here if there are also test cases under Lib/test/leakers.
+LEAKY_TESTS="test_(cmd_line|socket)"
+
+# Skip these tests altogether when looking for leaks.  These tests
+# do not need to be stored above in LEAKY_TESTS too.
+# test_compiler almost never finishes with the same number of refs
+# since it depends on other modules, skip it.
+# test_logging causes hangs, skip it.
+LEAKY_SKIPS="-x test_compiler test_logging"
+
+# Change this flag to "yes" for old releases to only update/build the docs.
 BUILD_DISABLED="yes"
 
 ## utility functions
@@ -71,7 +92,12 @@ update_status() {
 
 mail_on_failure() {
     if [ "$NUM_FAILURES" != "0" ]; then
-        mutt -s "$FAILURE_SUBJECT $1 ($NUM_FAILURES)" $FAILURE_MAILTO -c $FAILURE_CC < $2
+        dest=$FAILURE_MAILTO
+        # FAILURE_CC is optional.
+        if [ "$FAILURE_CC" != "" ]; then
+            dest="$dest -c $FAILURE_CC"
+        fi
+        mutt -s "$FAILURE_SUBJECT $1 ($NUM_FAILURES)" $dest < $2
     fi
 }
 
@@ -141,20 +167,32 @@ if [ $err = 0 -a "$BUILD_DISABLED" != "yes" ]; then
             make install >& build/$F
             update_status "Installing" "$F" $start
 
+            if [ ! -x $PYTHON ]; then
+                ln -s ${PYTHON}2.* $PYTHON
+            fi
+
             ## make and run basic tests
             F=make-test.out
             start=`current_time`
-            make test >& build/$F
-            NUM_FAILURES=`grep -ic " test failed:" build/$F`
+            $PYTHON $REGRTEST_ARGS >& build/$F
+            NUM_FAILURES=`grep -ic " failed:" build/$F`
             update_status "Testing basics ($NUM_FAILURES failures)" "$F" $start
-            ## FIXME: should mail since -uall below should find same problems
             mail_on_failure "basics" build/$F
 
+            F=make-test-opt.out
+            start=`current_time`
+            $PYTHON -O $REGRTEST_ARGS >& build/$F
+            NUM_FAILURES=`grep -ic " failed:" build/$F`
+            update_status "Testing opt ($NUM_FAILURES failures)" "$F" $start
+            mail_on_failure "opt" build/$F
+
             ## run the tests looking for leaks
             F=make-test-refleak.out
             start=`current_time`
-            ./python ./Lib/test/regrtest.py -R 4:3:$REFLOG -u network >& build/$F
-            NUM_FAILURES=`grep -ic leak $REFLOG`
+            ## ensure that the reflog exists so the grep doesn't fail
+            touch $REFLOG
+            $PYTHON $REGRTEST_ARGS -R 4:3:$REFLOG -u network $LEAKY_SKIPS >& build/$F
+            NUM_FAILURES=`egrep -vc "$LEAKY_TESTS" $REFLOG`
             update_status "Testing refleaks ($NUM_FAILURES failures)" "$F" $start
             mail_on_failure "refleak" $REFLOG
 
@@ -163,8 +201,8 @@ if [ $err = 0 -a "$BUILD_DISABLED" != "yes" ]; then
             start=`current_time`
             ## skip curses when running from cron since there's no terminal
             ## skip sound since it's not setup on the PSF box (/dev/dsp)
-            ./python -E -tt ./Lib/test/regrtest.py -uall -x test_curses test_linuxaudiodev test_ossaudiodev >& build/$F
-            NUM_FAILURES=`grep -ic fail build/$F`
+            $PYTHON $REGRTEST_ARGS -uall -x test_curses test_linuxaudiodev test_ossaudiodev >& build/$F
+            NUM_FAILURES=`grep -ic " failed:" build/$F`
             update_status "Testing all except curses and sound ($NUM_FAILURES failures)" "$F" $start
             mail_on_failure "all" build/$F
         fi
@@ -173,11 +211,22 @@ fi
 
 
 ## make doc
-cd Doc
+cd $DIR/Doc
 F="make-doc.out"
 start=`current_time`
-make >& ../build/$F
-err=$?
+# Doc/commontex/boilerplate.tex is expected to always have an outstanding
+# modification for the date.  When a release is cut, a conflict occurs.
+# This allows us to detect this problem and not try to build the docs
+# which will definitely fail with a conflict. 
+CONFLICTED_FILE=commontex/boilerplate.tex
+conflict_count=`grep -c "<<<" $CONFLICTED_FILE`
+if [ $conflict_count != 0 ]; then
+    echo "Conflict detected in $CONFLICTED_FILE.  Doc build skipped." > ../build/$F
+    err=1
+else
+    make >& ../build/$F
+    err=$?
+fi
 update_status "Making doc" "$F" $start
 if [ $err != 0 ]; then
     NUM_FAILURES=1
