@@ -1,7 +1,7 @@
 /*
  /  Author: Sam Rushing <rushing@nightmare.com>
  /  Hacked for Unix by AMK
- /  $Id: mmapmodule.c 65333 2008-07-31 17:04:32Z neal.norwitz $
+ /  $Id: mmapmodule.c 51474 2006-08-22 13:57:07Z neal.norwitz $
 
  / mmapmodule.cpp -- map a view of a file into memory
  /
@@ -9,15 +9,13 @@
  /   not all functions check range yet!!!
  /
  /
- / Note: This module currently only deals with 32-bit file
- /   sizes.
- /
  / This version of mmapmodule.c has been changed significantly
  / from the original mmapfile.c on which it was based.
  / The original version of mmapfile is maintained by Sam at
  / ftp://squirl.nightmare.com/pub/python/python-ext.
 */
 
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
 #ifndef MS_WINDOWS
@@ -52,7 +50,15 @@ my_getpagesize(void)
 #endif /* UNIX */
 
 #include <string.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif /* HAVE_SYS_TYPES_H */
+
+/* Prefer MAP_ANONYMOUS since MAP_ANON is deprecated according to man page. */
+#if !defined(MAP_ANONYMOUS) && defined(MAP_ANON)
+#  define MAP_ANONYMOUS MAP_ANON
+#endif
 
 static PyObject *mmap_module_error;
 
@@ -111,28 +117,26 @@ mmap_object_dealloc(mmap_object *m_obj)
 }
 
 static PyObject *
-mmap_close_method(mmap_object *self, PyObject *args)
+mmap_close_method(mmap_object *self, PyObject *unused)
 {
-        if (!PyArg_ParseTuple(args, ":close"))
-		return NULL;
 #ifdef MS_WINDOWS
 	/* For each resource we maintain, we need to check
-	   the value is valid, and if so, free the resource 
+	   the value is valid, and if so, free the resource
 	   and set the member value to an invalid value so
 	   the dealloc does not attempt to resource clearing
 	   again.
 	   TODO - should we check for errors in the close operations???
 	*/
 	if (self->data != NULL) {
-		UnmapViewOfFile (self->data);
+		UnmapViewOfFile(self->data);
 		self->data = NULL;
 	}
 	if (self->map_handle != INVALID_HANDLE_VALUE) {
-		CloseHandle (self->map_handle);
+		CloseHandle(self->map_handle);
 		self->map_handle = INVALID_HANDLE_VALUE;
 	}
 	if (self->file_handle != INVALID_HANDLE_VALUE) {
-		CloseHandle (self->file_handle);
+		CloseHandle(self->file_handle);
 		self->file_handle = INVALID_HANDLE_VALUE;
 	}
 #endif /* MS_WINDOWS */
@@ -146,15 +150,15 @@ mmap_close_method(mmap_object *self, PyObject *args)
 	}
 #endif
 
-	Py_INCREF (Py_None);
-	return (Py_None);
+	Py_INCREF(Py_None);
+	return Py_None;
 }
 
 #ifdef MS_WINDOWS
 #define CHECK_VALID(err)						\
 do {									\
     if (self->map_handle == INVALID_HANDLE_VALUE) {						\
-	PyErr_SetString (PyExc_ValueError, "mmap closed or invalid");	\
+	PyErr_SetString(PyExc_ValueError, "mmap closed or invalid");	\
 	return err;							\
     }									\
 } while (0)
@@ -164,7 +168,7 @@ do {									\
 #define CHECK_VALID(err)						\
 do {									\
     if (self->data == NULL) {						\
-	PyErr_SetString (PyExc_ValueError, "mmap closed or invalid");	\
+	PyErr_SetString(PyExc_ValueError, "mmap closed or invalid");	\
 	return err;							\
 	}								\
 } while (0)
@@ -172,24 +176,22 @@ do {									\
 
 static PyObject *
 mmap_read_byte_method(mmap_object *self,
-		      PyObject *args)
+		      PyObject *unused)
 {
 	CHECK_VALID(NULL);
-        if (!PyArg_ParseTuple(args, ":read_byte"))
-		return NULL;
 	if (self->pos < self->size) {
 	        char value = self->data[self->pos];
 		self->pos += 1;
 		return Py_BuildValue("c", value);
 	} else {
-		PyErr_SetString (PyExc_ValueError, "read byte out of range");
+		PyErr_SetString(PyExc_ValueError, "read byte out of range");
 		return NULL;
 	}
 }
 
 static PyObject *
 mmap_read_line_method(mmap_object *self,
-		      PyObject *args)
+		      PyObject *unused)
 {
 	char *start = self->data+self->pos;
 	char *eof = self->data+self->size;
@@ -197,8 +199,6 @@ mmap_read_line_method(mmap_object *self,
 	PyObject *result;
 
 	CHECK_VALID(NULL);
-        if (!PyArg_ParseTuple(args, ":readline"))
-		return NULL;
 
 	eol = memchr(start, '\n', self->size - self->pos);
 	if (!eol)
@@ -208,39 +208,39 @@ mmap_read_line_method(mmap_object *self,
 				   newline. */
 	result = PyString_FromStringAndSize(start, (eol - start));
 	self->pos += (eol - start);
-	return (result);
+	return result;
 }
 
 static PyObject *
 mmap_read_method(mmap_object *self,
 		 PyObject *args)
 {
-	long num_bytes;
+	Py_ssize_t num_bytes;
 	PyObject *result;
 
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple(args, "l:read", &num_bytes))
+	if (!PyArg_ParseTuple(args, "n:read", &num_bytes))
 		return(NULL);
 
 	/* silently 'adjust' out-of-range requests */
-	if (num_bytes > self->size - self->pos) {
+	if ((self->pos + num_bytes) > self->size) {
 		num_bytes -= (self->pos+num_bytes) - self->size;
 	}
 	result = Py_BuildValue("s#", self->data+self->pos, num_bytes);
-	self->pos += num_bytes;	 
-	return (result);
+	self->pos += num_bytes;
+	return result;
 }
 
 static PyObject *
 mmap_find_method(mmap_object *self,
 		 PyObject *args)
 {
-	long start = self->pos;
+	Py_ssize_t start = self->pos;
 	char *needle;
-	int len;
+	Py_ssize_t len;
 
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "s#|l:find", &needle, &len, &start)) {
+	if (!PyArg_ParseTuple(args, "s#|n:find", &needle, &len, &start)) {
 		return NULL;
 	} else {
 		char *p;
@@ -254,34 +254,32 @@ mmap_find_method(mmap_object *self,
 			start = self->size;
 
 		for (p = self->data + start; p + len <= e; ++p) {
-			int i;
+			Py_ssize_t i;
 			for (i = 0; i < len && needle[i] == p[i]; ++i)
 				/* nothing */;
 			if (i == len) {
-				return Py_BuildValue (
-					"l",
-					(long) (p - self->data));
+				return PyInt_FromSsize_t(p - self->data);
 			}
 		}
-		return Py_BuildValue ("l", (long) -1);
+		return PyInt_FromLong(-1);
 	}
 }
 
-static int 
+static int
 is_writeable(mmap_object *self)
 {
 	if (self->access != ACCESS_READ)
-		return 1; 
+		return 1;
 	PyErr_Format(PyExc_TypeError, "mmap can't modify a readonly memory map.");
 	return 0;
 }
 
-static int 
+static int
 is_resizeable(mmap_object *self)
 {
 	if ((self->access == ACCESS_WRITE) || (self->access == ACCESS_DEFAULT))
-		return 1; 
-	PyErr_Format(PyExc_TypeError, 
+		return 1;
+	PyErr_Format(PyExc_TypeError,
 		     "mmap can't resize a readonly or copy-on-write memory map.");
 	return 0;
 }
@@ -291,24 +289,24 @@ static PyObject *
 mmap_write_method(mmap_object *self,
 		  PyObject *args)
 {
-	int length;
+	Py_ssize_t length;
 	char *data;
 
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "s#:write", &data, &length))
+	if (!PyArg_ParseTuple(args, "s#:write", &data, &length))
 		return(NULL);
 
 	if (!is_writeable(self))
 		return NULL;
 
 	if ((self->pos + length) > self->size) {
-		PyErr_SetString (PyExc_ValueError, "data out of range");
+		PyErr_SetString(PyExc_ValueError, "data out of range");
 		return NULL;
 	}
-	memcpy (self->data+self->pos, data, length);
+	memcpy(self->data+self->pos, data, length);
 	self->pos = self->pos+length;
-	Py_INCREF (Py_None);
-	return (Py_None);
+	Py_INCREF(Py_None);
+	return Py_None;
 }
 
 static PyObject *
@@ -318,32 +316,41 @@ mmap_write_byte_method(mmap_object *self,
 	char value;
 
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "c:write_byte", &value))
+	if (!PyArg_ParseTuple(args, "c:write_byte", &value))
 		return(NULL);
 
 	if (!is_writeable(self))
 		return NULL;
 	*(self->data+self->pos) = value;
 	self->pos += 1;
-	Py_INCREF (Py_None);
-	return (Py_None);
+	Py_INCREF(Py_None);
+	return Py_None;
 }
- 
+
 static PyObject *
 mmap_size_method(mmap_object *self,
-		 PyObject *args)
+		 PyObject *unused)
 {
 	CHECK_VALID(NULL);
-        if (!PyArg_ParseTuple(args, ":size"))
-		return NULL;
 
 #ifdef MS_WINDOWS
 	if (self->file_handle != INVALID_HANDLE_VALUE) {
-		return (Py_BuildValue (
-			"l", (long)
-			GetFileSize (self->file_handle, NULL)));
+		DWORD low,high;
+		PY_LONG_LONG size;
+		low = GetFileSize(self->file_handle, &high);
+		if (low == INVALID_FILE_SIZE) {
+			/* It might be that the function appears to have failed,
+			   when indeed its size equals INVALID_FILE_SIZE */
+			DWORD error = GetLastError();
+			if (error != NO_ERROR)
+				return PyErr_SetFromWindowsErr(error);
+		}
+		if (!high && low < LONG_MAX)
+			return PyInt_FromLong((long)low);
+		size = (((PY_LONG_LONG)high)<<32) + low;
+		return PyLong_FromLongLong(size);
 	} else {
-		return (Py_BuildValue ("l", (long) self->size) );
+		return PyInt_FromSsize_t(self->size);
 	}
 #endif /* MS_WINDOWS */
 
@@ -354,7 +361,7 @@ mmap_size_method(mmap_object *self,
 			PyErr_SetFromErrno(mmap_module_error);
 			return NULL;
 		}
-		return (Py_BuildValue ("l", (long) buf.st_size) );
+		return PyInt_FromSsize_t(buf.st_size);
 	}
 #endif /* UNIX */
 }
@@ -372,40 +379,48 @@ static PyObject *
 mmap_resize_method(mmap_object *self,
 		   PyObject *args)
 {
-	unsigned long new_size;
+	Py_ssize_t new_size;
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "l:resize", &new_size) || 
+	if (!PyArg_ParseTuple(args, "n:resize", &new_size) ||
 	    !is_resizeable(self)) {
 		return NULL;
 #ifdef MS_WINDOWS
-	} else { 
+	} else {
 		DWORD dwErrCode = 0;
+		DWORD newSizeLow, newSizeHigh;
 		/* First, unmap the file view */
-		UnmapViewOfFile (self->data);
+		UnmapViewOfFile(self->data);
 		/* Close the mapping object */
-		CloseHandle (self->map_handle);
+		CloseHandle(self->map_handle);
 		/* Move to the desired EOF position */
-		SetFilePointer (self->file_handle,
-				new_size, NULL, FILE_BEGIN);
+#if SIZEOF_SIZE_T > 4
+		newSizeHigh = (DWORD)(new_size >> 32);
+		newSizeLow = (DWORD)(new_size & 0xFFFFFFFF);
+#else
+		newSizeHigh = 0;
+		newSizeLow = (DWORD)new_size;
+#endif
+		SetFilePointer(self->file_handle,
+			       newSizeLow, &newSizeHigh, FILE_BEGIN);
 		/* Change the size of the file */
-		SetEndOfFile (self->file_handle);
+		SetEndOfFile(self->file_handle);
 		/* Create another mapping object and remap the file view */
-		self->map_handle = CreateFileMapping (
+		self->map_handle = CreateFileMapping(
 			self->file_handle,
 			NULL,
 			PAGE_READWRITE,
-			0,
-			new_size,
+			newSizeHigh,
+			newSizeLow,
 			self->tagname);
 		if (self->map_handle != NULL) {
-			self->data = (char *) MapViewOfFile (self->map_handle,
-							     FILE_MAP_WRITE,
-							     0,
-							     0,
-							     0);
+			self->data = (char *) MapViewOfFile(self->map_handle,
+							    FILE_MAP_WRITE,
+							    0,
+							    0,
+							    0);
 			if (self->data != NULL) {
 				self->size = new_size;
-				Py_INCREF (Py_None);
+				Py_INCREF(Py_None);
 				return Py_None;
 			} else {
 				dwErrCode = GetLastError();
@@ -414,11 +429,11 @@ mmap_resize_method(mmap_object *self,
 			dwErrCode = GetLastError();
 		}
 		PyErr_SetFromWindowsErr(dwErrCode);
-		return (NULL);
+		return NULL;
 #endif /* MS_WINDOWS */
 
 #ifdef UNIX
-#ifndef HAVE_MREMAP 
+#ifndef HAVE_MREMAP
 	} else {
 		PyErr_SetString(PyExc_SystemError,
 				"mmap: resizing not available--no mremap()");
@@ -437,7 +452,7 @@ mmap_resize_method(mmap_object *self,
 #else
 		newmap = mremap(self->data, self->size, new_size, 0);
 #endif
-		if (newmap == (void *)-1) 
+		if (newmap == (void *)-1)
 		{
 			PyErr_SetFromErrno(mmap_module_error);
 			return NULL;
@@ -452,30 +467,27 @@ mmap_resize_method(mmap_object *self,
 }
 
 static PyObject *
-mmap_tell_method(mmap_object *self, PyObject *args)
+mmap_tell_method(mmap_object *self, PyObject *unused)
 {
 	CHECK_VALID(NULL);
-        if (!PyArg_ParseTuple(args, ":tell"))
-		return NULL;
-	return (Py_BuildValue ("l", (long) self->pos) );
+	return PyInt_FromSize_t(self->pos);
 }
 
 static PyObject *
 mmap_flush_method(mmap_object *self, PyObject *args)
 {
-	size_t offset	= 0;
-	size_t size = self->size;
+	Py_ssize_t offset = 0;
+	Py_ssize_t size = self->size;
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "|ll:flush", &offset, &size)) {
+	if (!PyArg_ParseTuple(args, "|nn:flush", &offset, &size))
 		return NULL;
-	} else if ((offset + size) > self->size) {
-		PyErr_SetString (PyExc_ValueError,
-				 "flush values out of range");
+	if ((size_t)(offset + size) > self->size) {
+		PyErr_SetString(PyExc_ValueError, "flush values out of range");
 		return NULL;
 	} else {
 #ifdef MS_WINDOWS
-		return (Py_BuildValue("l", (long)
-                                      FlushViewOfFile(self->data+offset, size)));
+		return PyInt_FromLong((long)
+                                      FlushViewOfFile(self->data+offset, size));
 #endif /* MS_WINDOWS */
 #ifdef UNIX
 		/* XXX semantics of return value? */
@@ -486,20 +498,20 @@ mmap_flush_method(mmap_object *self, PyObject *args)
 			PyErr_SetFromErrno(mmap_module_error);
 			return NULL;
 		}
-		return Py_BuildValue ("l", (long) 0);	
-#endif /* UNIX */   
+		return PyInt_FromLong(0);
+#endif /* UNIX */
 	}
 }
 
 static PyObject *
 mmap_seek_method(mmap_object *self, PyObject *args)
 {
-	int dist;
+	Py_ssize_t dist;
 	int how=0;
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "i|i:seek", &dist, &how)) {
-		return(NULL);
-	} else {
+	if (!PyArg_ParseTuple(args, "n|i:seek", &dist, &how))
+		return NULL;
+	else {
 		size_t where;
 		switch (how) {
 		case 0: /* relative to start */
@@ -508,29 +520,28 @@ mmap_seek_method(mmap_object *self, PyObject *args)
 			where = dist;
 			break;
 		case 1: /* relative to current position */
-			if ((int)self->pos + dist < 0)
+			if ((Py_ssize_t)self->pos + dist < 0)
 				goto onoutofrange;
 			where = self->pos + dist;
 			break;
 		case 2: /* relative to end */
-			if ((int)self->size + dist < 0)
+			if ((Py_ssize_t)self->size + dist < 0)
 				goto onoutofrange;
 			where = self->size + dist;
 			break;
 		default:
-			PyErr_SetString (PyExc_ValueError,
-					 "unknown seek type");
+			PyErr_SetString(PyExc_ValueError, "unknown seek type");
 			return NULL;
 		}
 		if (where > self->size)
 			goto onoutofrange;
 		self->pos = where;
-		Py_INCREF (Py_None);
-		return (Py_None);
+		Py_INCREF(Py_None);
+		return Py_None;
 	}
 
   onoutofrange:
-	PyErr_SetString (PyExc_ValueError, "seek out of range");
+	PyErr_SetString(PyExc_ValueError, "seek out of range");
 	return NULL;
 }
 
@@ -539,7 +550,7 @@ mmap_move_method(mmap_object *self, PyObject *args)
 {
 	unsigned long dest, src, count;
 	CHECK_VALID(NULL);
-	if (!PyArg_ParseTuple (args, "iii:move", &dest, &src, &count) ||
+	if (!PyArg_ParseTuple(args, "kkk:move", &dest, &src, &count) ||
 	    !is_writeable(self)) {
 		return NULL;
 	} else {
@@ -548,29 +559,29 @@ mmap_move_method(mmap_object *self, PyObject *args)
 			((src+count) > self->size)
 			/* dest will fit? */
 			|| (dest+count > self->size)) {
-			PyErr_SetString (PyExc_ValueError,
-					 "source or destination out of range");
+			PyErr_SetString(PyExc_ValueError,
+					"source or destination out of range");
 			return NULL;
 		} else {
-			memmove (self->data+dest, self->data+src, count);
-			Py_INCREF (Py_None);
+			memmove(self->data+dest, self->data+src, count);
+			Py_INCREF(Py_None);
 			return Py_None;
 		}
 	}
 }
 
 static struct PyMethodDef mmap_object_methods[] = {
-	{"close",	(PyCFunction) mmap_close_method,	METH_VARARGS},
+	{"close",	(PyCFunction) mmap_close_method,	METH_NOARGS},
 	{"find",	(PyCFunction) mmap_find_method,		METH_VARARGS},
 	{"flush",	(PyCFunction) mmap_flush_method,	METH_VARARGS},
 	{"move",	(PyCFunction) mmap_move_method,		METH_VARARGS},
 	{"read",	(PyCFunction) mmap_read_method,		METH_VARARGS},
-	{"read_byte",	(PyCFunction) mmap_read_byte_method,  	METH_VARARGS},
-	{"readline",	(PyCFunction) mmap_read_line_method,	METH_VARARGS},
+	{"read_byte",	(PyCFunction) mmap_read_byte_method,  	METH_NOARGS},
+	{"readline",	(PyCFunction) mmap_read_line_method,	METH_NOARGS},
 	{"resize",	(PyCFunction) mmap_resize_method,	METH_VARARGS},
 	{"seek",	(PyCFunction) mmap_seek_method,		METH_VARARGS},
-	{"size",	(PyCFunction) mmap_size_method,		METH_VARARGS},
-	{"tell",	(PyCFunction) mmap_tell_method,		METH_VARARGS},
+	{"size",	(PyCFunction) mmap_size_method,		METH_NOARGS},
+	{"tell",	(PyCFunction) mmap_tell_method,		METH_NOARGS},
 	{"write",	(PyCFunction) mmap_write_method,	METH_VARARGS},
 	{"write_byte",	(PyCFunction) mmap_write_byte_method,	METH_VARARGS},
 	{NULL,	   NULL}       /* sentinel */
@@ -578,11 +589,11 @@ static struct PyMethodDef mmap_object_methods[] = {
 
 /* Functions for treating an mmap'ed file as a buffer */
 
-static int
-mmap_buffer_getreadbuf(mmap_object *self, int index, const void **ptr)
+static Py_ssize_t
+mmap_buffer_getreadbuf(mmap_object *self, Py_ssize_t index, const void **ptr)
 {
 	CHECK_VALID(-1);
-	if ( index != 0 ) {
+	if (index != 0) {
 		PyErr_SetString(PyExc_SystemError,
 				"Accessing non-existent mmap segment");
 		return -1;
@@ -591,11 +602,11 @@ mmap_buffer_getreadbuf(mmap_object *self, int index, const void **ptr)
 	return self->size;
 }
 
-static int
-mmap_buffer_getwritebuf(mmap_object *self, int index, const void **ptr)
-{  
+static Py_ssize_t
+mmap_buffer_getwritebuf(mmap_object *self, Py_ssize_t index, const void **ptr)
+{
 	CHECK_VALID(-1);
-	if ( index != 0 ) {
+	if (index != 0) {
 		PyErr_SetString(PyExc_SystemError,
 				"Accessing non-existent mmap segment");
 		return -1;
@@ -606,19 +617,19 @@ mmap_buffer_getwritebuf(mmap_object *self, int index, const void **ptr)
 	return self->size;
 }
 
-static int
-mmap_buffer_getsegcount(mmap_object *self, int *lenp)
+static Py_ssize_t
+mmap_buffer_getsegcount(mmap_object *self, Py_ssize_t *lenp)
 {
 	CHECK_VALID(-1);
-	if (lenp) 
+	if (lenp)
 		*lenp = self->size;
 	return 1;
 }
 
-static int
-mmap_buffer_getcharbuffer(mmap_object *self, int index, const void **ptr)
+static Py_ssize_t
+mmap_buffer_getcharbuffer(mmap_object *self, Py_ssize_t index, const void **ptr)
 {
-	if ( index != 0 ) {
+	if (index != 0) {
 		PyErr_SetString(PyExc_SystemError,
 				"accessing non-existent buffer segment");
 		return -1;
@@ -630,10 +641,10 @@ mmap_buffer_getcharbuffer(mmap_object *self, int index, const void **ptr)
 static PyObject *
 mmap_object_getattr(mmap_object *self, char *name)
 {
-	return Py_FindMethod (mmap_object_methods, (PyObject *)self, name);
+	return Py_FindMethod(mmap_object_methods, (PyObject *)self, name);
 }
 
-static int
+static Py_ssize_t
 mmap_length(mmap_object *self)
 {
 	CHECK_VALID(-1);
@@ -641,7 +652,7 @@ mmap_length(mmap_object *self)
 }
 
 static PyObject *
-mmap_item(mmap_object *self, int i)
+mmap_item(mmap_object *self, Py_ssize_t i)
 {
 	CHECK_VALID(NULL);
 	if (i < 0 || (size_t)i >= self->size) {
@@ -652,7 +663,7 @@ mmap_item(mmap_object *self, int i)
 }
 
 static PyObject *
-mmap_slice(mmap_object *self, int ilow, int ihigh)
+mmap_slice(mmap_object *self, Py_ssize_t ilow, Py_ssize_t ihigh)
 {
 	CHECK_VALID(NULL);
 	if (ilow < 0)
@@ -665,7 +676,7 @@ mmap_slice(mmap_object *self, int ilow, int ihigh)
 		ihigh = ilow;
 	else if ((size_t)ihigh > self->size)
 		ihigh = self->size;
-    
+
 	return PyString_FromStringAndSize(self->data + ilow, ihigh-ilow);
 }
 
@@ -679,7 +690,7 @@ mmap_concat(mmap_object *self, PyObject *bb)
 }
 
 static PyObject *
-mmap_repeat(mmap_object *self, int n)
+mmap_repeat(mmap_object *self, Py_ssize_t n)
 {
 	CHECK_VALID(NULL);
 	PyErr_SetString(PyExc_SystemError,
@@ -688,7 +699,7 @@ mmap_repeat(mmap_object *self, int n)
 }
 
 static int
-mmap_ass_slice(mmap_object *self, int ilow, int ihigh, PyObject *v)
+mmap_ass_slice(mmap_object *self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject *v)
 {
 	const char *buf;
 
@@ -703,19 +714,19 @@ mmap_ass_slice(mmap_object *self, int ilow, int ihigh, PyObject *v)
 		ihigh = ilow;
 	else if ((size_t)ihigh > self->size)
 		ihigh = self->size;
-    
+
 	if (v == NULL) {
 		PyErr_SetString(PyExc_TypeError,
 				"mmap object doesn't support slice deletion");
 		return -1;
 	}
 	if (! (PyString_Check(v)) ) {
-		PyErr_SetString(PyExc_IndexError, 
+		PyErr_SetString(PyExc_IndexError,
 				"mmap slice assignment must be a string");
 		return -1;
 	}
-	if ( PyString_Size(v) != (ihigh - ilow) ) {
-		PyErr_SetString(PyExc_IndexError, 
+	if (PyString_Size(v) != (ihigh - ilow)) {
+		PyErr_SetString(PyExc_IndexError,
 				"mmap slice assignment is wrong size");
 		return -1;
 	}
@@ -727,10 +738,10 @@ mmap_ass_slice(mmap_object *self, int ilow, int ihigh, PyObject *v)
 }
 
 static int
-mmap_ass_item(mmap_object *self, int i, PyObject *v)
+mmap_ass_item(mmap_object *self, Py_ssize_t i, PyObject *v)
 {
 	const char *buf;
- 
+
 	CHECK_VALID(-1);
 	if (i < 0 || (size_t)i >= self->size) {
 		PyErr_SetString(PyExc_IndexError, "mmap index out of range");
@@ -742,7 +753,7 @@ mmap_ass_item(mmap_object *self, int i, PyObject *v)
 		return -1;
 	}
 	if (! (PyString_Check(v) && PyString_Size(v)==1) ) {
-		PyErr_SetString(PyExc_IndexError, 
+		PyErr_SetString(PyExc_IndexError,
 				"mmap assignment must be single-character string");
 		return -1;
 	}
@@ -754,20 +765,20 @@ mmap_ass_item(mmap_object *self, int i, PyObject *v)
 }
 
 static PySequenceMethods mmap_as_sequence = {
-	(inquiry)mmap_length,		       /*sq_length*/
+	(lenfunc)mmap_length,		       /*sq_length*/
 	(binaryfunc)mmap_concat,	       /*sq_concat*/
-	(intargfunc)mmap_repeat,	       /*sq_repeat*/
-	(intargfunc)mmap_item,		       /*sq_item*/
-	(intintargfunc)mmap_slice,	       /*sq_slice*/
-	(intobjargproc)mmap_ass_item,	       /*sq_ass_item*/
-	(intintobjargproc)mmap_ass_slice,      /*sq_ass_slice*/
+	(ssizeargfunc)mmap_repeat,	       /*sq_repeat*/
+	(ssizeargfunc)mmap_item,		       /*sq_item*/
+	(ssizessizeargfunc)mmap_slice,	       /*sq_slice*/
+	(ssizeobjargproc)mmap_ass_item,	       /*sq_ass_item*/
+	(ssizessizeobjargproc)mmap_ass_slice,      /*sq_ass_slice*/
 };
 
 static PyBufferProcs mmap_as_buffer = {
-	(getreadbufferproc)mmap_buffer_getreadbuf,
-	(getwritebufferproc)mmap_buffer_getwritebuf,
-	(getsegcountproc)mmap_buffer_getsegcount,
-	(getcharbufferproc)mmap_buffer_getcharbuffer,
+	(readbufferproc)mmap_buffer_getreadbuf,
+	(writebufferproc)mmap_buffer_getwritebuf,
+	(segcountproc)mmap_buffer_getsegcount,
+	(charbufferproc)mmap_buffer_getcharbuffer,
 };
 
 static PyTypeObject mmap_object_type = {
@@ -799,60 +810,28 @@ static PyTypeObject mmap_object_type = {
 
 /* extract the map size from the given PyObject
 
-   The map size is restricted to [0, INT_MAX] because this is the current
-   Python limitation on object sizes. Although the mmap object *could* handle
-   a larger map size, there is no point because all the useful operations
-   (len(), slicing(), sequence indexing) are limited by a C int.
-
    Returns -1 on error, with an appropriate Python exception raised. On
    success, the map size is returned. */
-static int
+static Py_ssize_t
 _GetMapSize(PyObject *o)
 {
-	if (PyInt_Check(o)) {
-		long i = PyInt_AsLong(o);
-		if (PyErr_Occurred())
+	if (PyIndex_Check(o)) {
+		Py_ssize_t i = PyNumber_AsSsize_t(o, PyExc_OverflowError);
+		if (i==-1 && PyErr_Occurred()) 
+			return -1;
+		if (i < 0) {	 
+			PyErr_SetString(PyExc_OverflowError,
+					"memory mapped size must be positive");
 			return -1;
-		if (i < 0)
-			goto onnegoverflow;
-		if (i > INT_MAX)
-			goto onposoverflow;
-		return (int)i;
-	}
-	else if (PyLong_Check(o)) {
-		long i = PyLong_AsLong(o);
-		if (PyErr_Occurred()) {
-			/* yes negative overflow is mistaken for positive overflow
-			   but not worth the trouble to check sign of 'i' */
-			if (PyErr_ExceptionMatches(PyExc_OverflowError))
-				goto onposoverflow;
-			else
-				return -1;
 		}
-		if (i < 0)
-			goto onnegoverflow;
-		if (i > INT_MAX)
-			goto onposoverflow;
-		return (int)i;
-	}
-	else {
-		PyErr_SetString(PyExc_TypeError,
-				"map size must be an integral value");
-		return -1;
+		return i;
 	}
 
-  onnegoverflow:
-	PyErr_SetString(PyExc_OverflowError,
-			"memory mapped size must be positive");
-	return -1;
-
-  onposoverflow:
-	PyErr_SetString(PyExc_OverflowError,
-			"memory mapped size is too large (limited by C int)");
+	PyErr_SetString(PyExc_TypeError, "map size must be an integral value");
 	return -1;
 }
 
-#ifdef UNIX 
+#ifdef UNIX
 static PyObject *
 new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 {
@@ -861,25 +840,27 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 #endif
 	mmap_object *m_obj;
 	PyObject *map_size_obj = NULL;
-	int map_size;
+	Py_ssize_t map_size;
 	int fd, flags = MAP_SHARED, prot = PROT_WRITE | PROT_READ;
-	access_mode access = ACCESS_DEFAULT;
-	char *keywords[] = {"fileno", "length", 
-			    "flags", "prot", 
-			    "access", NULL};
+	int devzero = -1;
+	int access = (int)ACCESS_DEFAULT;
+	static char *keywords[] = {"fileno", "length",
+                                         "flags", "prot",
+                                         "access", NULL};
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwdict, "iO|iii", keywords, 
-					 &fd, &map_size_obj, &flags, &prot, &access))
+	if (!PyArg_ParseTupleAndKeywords(args, kwdict, "iO|iii", keywords,
+					 &fd, &map_size_obj, &flags, &prot,
+                                         &access))
 		return NULL;
 	map_size = _GetMapSize(map_size_obj);
 	if (map_size < 0)
 		return NULL;
 
-	if ((access != ACCESS_DEFAULT) && 
-	    ((flags != MAP_SHARED) || ( prot != (PROT_WRITE | PROT_READ))))
-		return PyErr_Format(PyExc_ValueError, 
+	if ((access != (int)ACCESS_DEFAULT) &&
+	    ((flags != MAP_SHARED) || (prot != (PROT_WRITE | PROT_READ))))
+		return PyErr_Format(PyExc_ValueError,
 				    "mmap can't specify both access and flags, prot.");
-	switch(access) {
+	switch ((access_mode)access) {
 	case ACCESS_READ:
 		flags = MAP_SHARED;
 		prot = PROT_READ;
@@ -892,11 +873,11 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 		flags = MAP_PRIVATE;
 		prot = PROT_READ | PROT_WRITE;
 		break;
-	case ACCESS_DEFAULT: 
+	case ACCESS_DEFAULT:
 		/* use the specified or default values of flags and prot */
 		break;
 	default:
-		return PyErr_Format(PyExc_ValueError, 
+		return PyErr_Format(PyExc_ValueError,
 				    "mmap invalid access parameter.");
 	}
 
@@ -905,20 +886,39 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 	/* on OpenVMS we must ensure that all bytes are written to the file */
 	fsync(fd);
 #  endif
-	if (fstat(fd, &st) == 0 && S_ISREG(st.st_mode) &&
-	    (size_t)map_size > st.st_size) {
-		PyErr_SetString(PyExc_ValueError, 
-				"mmap length is greater than file size");
-		return NULL;
+	if (fstat(fd, &st) == 0 && S_ISREG(st.st_mode)) {
+		if (map_size == 0) {
+			map_size = st.st_size;
+		} else if ((size_t)map_size > st.st_size) {
+			PyErr_SetString(PyExc_ValueError,
+					"mmap length is greater than file size");
+			return NULL;
+		}
 	}
 #endif
-	m_obj = PyObject_New (mmap_object, &mmap_object_type);
+	m_obj = PyObject_New(mmap_object, &mmap_object_type);
 	if (m_obj == NULL) {return NULL;}
 	m_obj->data = NULL;
 	m_obj->size = (size_t) map_size;
 	m_obj->pos = (size_t) 0;
 	if (fd == -1) {
 		m_obj->fd = -1;
+		/* Assume the caller wants to map anonymous memory.
+		   This is the same behaviour as Windows.  mmap.mmap(-1, size)
+		   on both Windows and Unix map anonymous memory.
+		*/
+#ifdef MAP_ANONYMOUS
+		/* BSD way to map anonymous memory */
+		flags |= MAP_ANONYMOUS;
+#else
+		/* SVR4 method to map anonymous memory is to open /dev/zero */
+		fd = devzero = open("/dev/zero", O_RDWR);
+		if (devzero == -1) {
+			Py_DECREF(m_obj);
+			PyErr_SetFromErrno(mmap_module_error);
+			return NULL;
+		}
+#endif
 	} else {
 		m_obj->fd = dup(fd);
 		if (m_obj->fd == -1) {
@@ -928,16 +928,21 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 		}
 	}
 
-	m_obj->data = mmap(NULL, map_size, 
+	m_obj->data = mmap(NULL, map_size,
 			   prot, flags,
 			   fd, 0);
+
+	if (devzero != -1) {
+		close(devzero);
+	}
+
 	if (m_obj->data == (char *)-1) {
 	        m_obj->data = NULL;
 		Py_DECREF(m_obj);
 		PyErr_SetFromErrno(mmap_module_error);
 		return NULL;
 	}
-	m_obj->access = access;
+	m_obj->access = (access_mode)access;
 	return (PyObject *)m_obj;
 }
 #endif /* UNIX */
@@ -948,24 +953,26 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 {
 	mmap_object *m_obj;
 	PyObject *map_size_obj = NULL;
-	int map_size;
+	Py_ssize_t map_size;
+	DWORD size_hi;	/* upper 32 bits of m_obj->size */
+	DWORD size_lo;	/* lower 32 bits of m_obj->size */
 	char *tagname = "";
 	DWORD dwErr = 0;
 	int fileno;
 	HANDLE fh = 0;
-	access_mode   access = ACCESS_DEFAULT;
+	int access = (access_mode)ACCESS_DEFAULT;
 	DWORD flProtect, dwDesiredAccess;
-	char *keywords[] = { "fileno", "length", 
-			     "tagname", 
-			     "access", NULL };
+	static char *keywords[] = { "fileno", "length",
+                                          "tagname",
+                                          "access", NULL };
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwdict, "iO|zi", keywords,
-					 &fileno, &map_size_obj, 
+					 &fileno, &map_size_obj,
 					 &tagname, &access)) {
 		return NULL;
 	}
 
-	switch(access) {
+	switch((access_mode)access) {
 	case ACCESS_READ:
 		flProtect = PAGE_READONLY;
 		dwDesiredAccess = FILE_MAP_READ;
@@ -979,16 +986,23 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 		dwDesiredAccess = FILE_MAP_COPY;
 		break;
 	default:
-		return PyErr_Format(PyExc_ValueError, 
+		return PyErr_Format(PyExc_ValueError,
 				    "mmap invalid access parameter.");
 	}
 
 	map_size = _GetMapSize(map_size_obj);
 	if (map_size < 0)
 		return NULL;
-	
-	/* if an actual filename has been specified */
-	if (fileno != 0) {
+
+	/* assume -1 and 0 both mean invalid filedescriptor
+	   to 'anonymously' map memory.
+	   XXX: fileno == 0 is a valid fd, but was accepted prior to 2.5.
+	   XXX: Should this code be added?
+	   if (fileno == 0)
+	   	PyErr_Warn(PyExc_DeprecationWarning,
+			   "don't use 0 for anonymous memory");
+	 */
+	if (fileno != -1 && fileno != 0) {
 		fh = (HANDLE)_get_osfhandle(fileno);
 		if (fh==(HANDLE)-1) {
 			PyErr_SetFromErrno(mmap_module_error);
@@ -998,8 +1012,8 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 		lseek(fileno, 0, SEEK_SET);
 	}
 
-	m_obj = PyObject_New (mmap_object, &mmap_object_type);
-	if (m_obj==NULL)
+	m_obj = PyObject_New(mmap_object, &mmap_object_type);
+	if (m_obj == NULL)
 		return NULL;
 	/* Set every field to an invalid marker, so we can safely
 	   destruct the object in the face of failure */
@@ -1025,7 +1039,25 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 			return NULL;
 		}
 		if (!map_size) {
-			m_obj->size = GetFileSize (fh, NULL);
+			DWORD low,high;
+			low = GetFileSize(fh, &high);
+			/* low might just happen to have the value INVALID_FILE_SIZE;
+    			   so we need to check the last error also. */
+			if (low == INVALID_FILE_SIZE &&
+			    (dwErr = GetLastError()) != NO_ERROR) {
+				Py_DECREF(m_obj);
+				return PyErr_SetFromWindowsErr(dwErr);
+			}	
+				    
+#if SIZEOF_SIZE_T > 4
+			m_obj->size = (((size_t)high)<<32) + low;
+#else
+			if (high)
+				/* File is too large to map completely */
+				m_obj->size = (size_t)-1;
+			else
+				m_obj->size = low;
+#endif
 		} else {
 			m_obj->size = map_size;
 		}
@@ -1050,40 +1082,59 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 	else
 		m_obj->tagname = NULL;
 
-	m_obj->access = access;
-	m_obj->map_handle = CreateFileMapping (m_obj->file_handle,
-					       NULL,
-					       flProtect,
-					       0,
-					       m_obj->size,
-					       m_obj->tagname);
+	m_obj->access = (access_mode)access;
+	/* DWORD is a 4-byte int.  If we're on a box where size_t consumes
+	 * more than 4 bytes, we need to break it apart.  Else (size_t
+	 * consumes 4 bytes), C doesn't define what happens if we shift
+	 * right by 32, so we need different code.
+	 */
+#if SIZEOF_SIZE_T > 4
+	size_hi = (DWORD)(m_obj->size >> 32);
+	size_lo = (DWORD)(m_obj->size & 0xFFFFFFFF);
+#else
+	size_hi = 0;
+	size_lo = (DWORD)m_obj->size;
+#endif
+	m_obj->map_handle = CreateFileMapping(m_obj->file_handle,
+					      NULL,
+					      flProtect,
+					      size_hi,
+					      size_lo,
+					      m_obj->tagname);
 	if (m_obj->map_handle != NULL) {
-		m_obj->data = (char *) MapViewOfFile (m_obj->map_handle,
-						      dwDesiredAccess,
-						      0,
-						      0,
-						      0);
-		if (m_obj->data != NULL) {
-			return ((PyObject *) m_obj);
-		} else {
+		m_obj->data = (char *) MapViewOfFile(m_obj->map_handle,
+						     dwDesiredAccess,
+						     0,
+						     0,
+						     0);
+		if (m_obj->data != NULL)
+			return (PyObject *)m_obj;
+		else
 			dwErr = GetLastError();
-		}
-	} else {
+	} else
 		dwErr = GetLastError();
-	}
 	Py_DECREF(m_obj);
 	PyErr_SetFromWindowsErr(dwErr);
-	return (NULL);
+	return NULL;
 }
 #endif /* MS_WINDOWS */
 
 /* List of functions exported by this module */
 static struct PyMethodDef mmap_functions[] = {
-	{"mmap",	(PyCFunction) new_mmap_object, 
+	{"mmap",	(PyCFunction) new_mmap_object,
 	 METH_VARARGS|METH_KEYWORDS},
 	{NULL,		NULL}	     /* Sentinel */
 };
 
+static void
+setint(PyObject *d, const char *name, long value)
+{
+	PyObject *o = PyInt_FromLong(value);
+	if (o && PyDict_SetItemString(d, name, o) == 0) {
+		Py_DECREF(o);
+	}
+}
+
 PyMODINIT_FUNC
 	initmmap(void)
 {
@@ -1092,51 +1143,44 @@ PyMODINIT_FUNC
 	/* Patch the object type */
 	mmap_object_type.ob_type = &PyType_Type;
 
-	module = Py_InitModule ("mmap", mmap_functions);
+	module = Py_InitModule("mmap", mmap_functions);
 	if (module == NULL)
 		return;
-	dict = PyModule_GetDict (module);
+	dict = PyModule_GetDict(module);
+	if (!dict)
+		return;
 	mmap_module_error = PyExc_EnvironmentError;
-	Py_INCREF(mmap_module_error);
-	PyDict_SetItemString (dict, "error", mmap_module_error);
+	PyDict_SetItemString(dict, "error", mmap_module_error);
 #ifdef PROT_EXEC
-	PyDict_SetItemString (dict, "PROT_EXEC", PyInt_FromLong(PROT_EXEC) );
+	setint(dict, "PROT_EXEC", PROT_EXEC);
 #endif
 #ifdef PROT_READ
-	PyDict_SetItemString (dict, "PROT_READ", PyInt_FromLong(PROT_READ) );
+	setint(dict, "PROT_READ", PROT_READ);
 #endif
 #ifdef PROT_WRITE
-	PyDict_SetItemString (dict, "PROT_WRITE", PyInt_FromLong(PROT_WRITE) );
+	setint(dict, "PROT_WRITE", PROT_WRITE);
 #endif
 
 #ifdef MAP_SHARED
-	PyDict_SetItemString (dict, "MAP_SHARED", PyInt_FromLong(MAP_SHARED) );
+	setint(dict, "MAP_SHARED", MAP_SHARED);
 #endif
 #ifdef MAP_PRIVATE
-	PyDict_SetItemString (dict, "MAP_PRIVATE",
-			      PyInt_FromLong(MAP_PRIVATE) );
+	setint(dict, "MAP_PRIVATE", MAP_PRIVATE);
 #endif
 #ifdef MAP_DENYWRITE
-	PyDict_SetItemString (dict, "MAP_DENYWRITE",
-			      PyInt_FromLong(MAP_DENYWRITE) );
+	setint(dict, "MAP_DENYWRITE", MAP_DENYWRITE);
 #endif
 #ifdef MAP_EXECUTABLE
-	PyDict_SetItemString (dict, "MAP_EXECUTABLE",
-			      PyInt_FromLong(MAP_EXECUTABLE) );
+	setint(dict, "MAP_EXECUTABLE", MAP_EXECUTABLE);
 #endif
-#ifdef MAP_ANON
-	PyDict_SetItemString (dict, "MAP_ANON", PyInt_FromLong(MAP_ANON) );
-	PyDict_SetItemString (dict, "MAP_ANONYMOUS",
-			      PyInt_FromLong(MAP_ANON) );
+#ifdef MAP_ANONYMOUS
+	setint(dict, "MAP_ANON", MAP_ANONYMOUS);
+	setint(dict, "MAP_ANONYMOUS", MAP_ANONYMOUS);
 #endif
 
-	PyDict_SetItemString (dict, "PAGESIZE",
-			      PyInt_FromLong( (long)my_getpagesize() ) );
+	setint(dict, "PAGESIZE", (long)my_getpagesize());
 
-	PyDict_SetItemString (dict, "ACCESS_READ",	
-			      PyInt_FromLong(ACCESS_READ));
-	PyDict_SetItemString (dict, "ACCESS_WRITE", 
-			      PyInt_FromLong(ACCESS_WRITE));
-	PyDict_SetItemString (dict, "ACCESS_COPY",	
-			      PyInt_FromLong(ACCESS_COPY));
+	setint(dict, "ACCESS_READ", ACCESS_READ);
+	setint(dict, "ACCESS_WRITE", ACCESS_WRITE);
+	setint(dict, "ACCESS_COPY", ACCESS_COPY);
 }
