@@ -823,109 +823,109 @@ pure_quantify_add_annotation(PyObject *self, PyObject *args)
 static struct PyMethodDef
 pure_methods[] = {
 #ifdef COMMON_PURE_FUNCTIONS
-    {"pure_logfile_printf",            pure_pure_logfile_printf,            1},
-    {"pure_printf",                    pure_pure_printf,                    1},
-    {"pure_printf_with_banner",        pure_pure_printf_with_banner,        1},
+    {"pure_logfile_printf",            pure_pure_logfile_printf,            METH_VARARGS},
+    {"pure_printf",                    pure_pure_printf,                    METH_VARARGS},
+    {"pure_printf_with_banner",        pure_pure_printf_with_banner,        METH_VARARGS},
 #endif /* COMMON_PURE_FUNCTIONS */
 #ifdef PURIFY_H
-    {"purify_all_inuse",               pure_purify_all_inuse,               1},
-    {"purify_all_leaks",               pure_purify_all_leaks,               1},
-    {"purify_new_inuse",               pure_purify_new_inuse,               1},
-    {"purify_new_leaks",               pure_purify_new_leaks,               1},
-    {"purify_clear_inuse",             pure_purify_clear_inuse,             1},
-    {"purify_clear_leaks",             pure_purify_clear_leaks,             1},
-    {"purify_all_fds_inuse",           pure_purify_all_fds_inuse,           1},
-    {"purify_new_fds_inuse",           pure_purify_new_fds_inuse,           1},
+    {"purify_all_inuse",               pure_purify_all_inuse,               METH_VARARGS},
+    {"purify_all_leaks",               pure_purify_all_leaks,               METH_VARARGS},
+    {"purify_new_inuse",               pure_purify_new_inuse,               METH_VARARGS},
+    {"purify_new_leaks",               pure_purify_new_leaks,               METH_VARARGS},
+    {"purify_clear_inuse",             pure_purify_clear_inuse,             METH_VARARGS},
+    {"purify_clear_leaks",             pure_purify_clear_leaks,             METH_VARARGS},
+    {"purify_all_fds_inuse",           pure_purify_all_fds_inuse,           METH_VARARGS},
+    {"purify_new_fds_inuse",           pure_purify_new_fds_inuse,           METH_VARARGS},
     /* see purify.h */
-    {"purify_logfile_printf",          pure_pure_logfile_printf,            1},
-    {"purify_printf",                  pure_pure_printf,                    1},
-    {"purify_printf_with_banner",      pure_pure_printf_with_banner,        1},
+    {"purify_logfile_printf",          pure_pure_logfile_printf,            METH_VARARGS},
+    {"purify_printf",                  pure_pure_printf,                    METH_VARARGS},
+    {"purify_printf_with_banner",      pure_pure_printf_with_banner,        METH_VARARGS},
     /**/
-    {"purify_printf_with_call_chain",  pure_purify_printf_with_call_chain,  1},
-    {"purify_set_pool_id",             pure_purify_set_pool_id,             1},
-    {"purify_get_pool_id",             pure_purify_get_pool_id,             1},
-    {"purify_set_user_data",           pure_purify_set_user_data,           1},
-    {"purify_get_user_data",           pure_purify_get_user_data,           1},
-    {"purify_map_pool",                pure_purify_map_pool,                1},
-    {"purify_map_pool_id",             pure_purify_map_pool_id,             1},
-    {"purify_new_messages",            pure_purify_new_messages,            1},
-    {"purify_all_messages",            pure_purify_all_messages,            1},
-    {"purify_clear_messages",          pure_purify_clear_messages,          1},
-    {"purify_clear_new_messages",      pure_purify_clear_new_messages,      1},
-    {"purify_start_batch",             pure_purify_start_batch,             1},
-    {"purify_start_batch_show_first",  pure_purify_start_batch_show_first,  1},
-    {"purify_stop_batch",              pure_purify_stop_batch,              1},
-    {"purify_name_thread",             pure_purify_name_thread,             1},
-    {"purify_watch",                   pure_purify_watch,                   1},
-    {"purify_watch_1",                 pure_purify_watch_1,                 1},
-    {"purify_watch_2",                 pure_purify_watch_2,                 1},
-    {"purify_watch_4",                 pure_purify_watch_4,                 1},
-    {"purify_watch_8",                 pure_purify_watch_8,                 1},
-    {"purify_watch_w_1",               pure_purify_watch_w_1,               1},
-    {"purify_watch_w_2",               pure_purify_watch_w_2,               1},
-    {"purify_watch_w_4",               pure_purify_watch_w_4,               1},
-    {"purify_watch_w_8",               pure_purify_watch_w_8,               1},
-    {"purify_watch_r_1",               pure_purify_watch_r_1,               1},
-    {"purify_watch_r_2",               pure_purify_watch_r_2,               1},
-    {"purify_watch_r_4",               pure_purify_watch_r_4,               1},
-    {"purify_watch_r_8",               pure_purify_watch_r_8,               1},
-    {"purify_watch_rw_1",              pure_purify_watch_rw_1,              1},
-    {"purify_watch_rw_2",              pure_purify_watch_rw_2,              1},
-    {"purify_watch_rw_4",              pure_purify_watch_rw_4,              1},
-    {"purify_watch_rw_8",              pure_purify_watch_rw_8,              1},
-    {"purify_watch_n",                 pure_purify_watch_n,                 1},
-    {"purify_watch_info",              pure_purify_watch_info,              1},
-    {"purify_watch_remove",            pure_purify_watch_remove,            1},
-    {"purify_watch_remove_all",        pure_purify_watch_remove_all,        1},
-    {"purify_describe",                pure_purify_describe,                1},
-    {"purify_what_colors",             pure_purify_what_colors,             1},
-    {"purify_is_running",              pure_purify_is_running,              1},
-    {"purify_assert_is_readable",      pure_purify_assert_is_readable,      1},
-    {"purify_assert_is_writable",      pure_purify_assert_is_writable,      1},
+    {"purify_printf_with_call_chain",  pure_purify_printf_with_call_chain,  METH_VARARGS},
+    {"purify_set_pool_id",             pure_purify_set_pool_id,             METH_VARARGS},
+    {"purify_get_pool_id",             pure_purify_get_pool_id,             METH_VARARGS},
+    {"purify_set_user_data",           pure_purify_set_user_data,           METH_VARARGS},
+    {"purify_get_user_data",           pure_purify_get_user_data,           METH_VARARGS},
+    {"purify_map_pool",                pure_purify_map_pool,                METH_VARARGS},
+    {"purify_map_pool_id",             pure_purify_map_pool_id,             METH_VARARGS},
+    {"purify_new_messages",            pure_purify_new_messages,            METH_VARARGS},
+    {"purify_all_messages",            pure_purify_all_messages,            METH_VARARGS},
+    {"purify_clear_messages",          pure_purify_clear_messages,          METH_VARARGS},
+    {"purify_clear_new_messages",      pure_purify_clear_new_messages,      METH_VARARGS},
+    {"purify_start_batch",             pure_purify_start_batch,             METH_VARARGS},
+    {"purify_start_batch_show_first",  pure_purify_start_batch_show_first,  METH_VARARGS},
+    {"purify_stop_batch",              pure_purify_stop_batch,              METH_VARARGS},
+    {"purify_name_thread",             pure_purify_name_thread,             METH_VARARGS},
+    {"purify_watch",                   pure_purify_watch,                   METH_VARARGS},
+    {"purify_watch_1",                 pure_purify_watch_1,                 METH_VARARGS},
+    {"purify_watch_2",                 pure_purify_watch_2,                 METH_VARARGS},
+    {"purify_watch_4",                 pure_purify_watch_4,                 METH_VARARGS},
+    {"purify_watch_8",                 pure_purify_watch_8,                 METH_VARARGS},
+    {"purify_watch_w_1",               pure_purify_watch_w_1,               METH_VARARGS},
+    {"purify_watch_w_2",               pure_purify_watch_w_2,               METH_VARARGS},
+    {"purify_watch_w_4",               pure_purify_watch_w_4,               METH_VARARGS},
+    {"purify_watch_w_8",               pure_purify_watch_w_8,               METH_VARARGS},
+    {"purify_watch_r_1",               pure_purify_watch_r_1,               METH_VARARGS},
+    {"purify_watch_r_2",               pure_purify_watch_r_2,               METH_VARARGS},
+    {"purify_watch_r_4",               pure_purify_watch_r_4,               METH_VARARGS},
+    {"purify_watch_r_8",               pure_purify_watch_r_8,               METH_VARARGS},
+    {"purify_watch_rw_1",              pure_purify_watch_rw_1,              METH_VARARGS},
+    {"purify_watch_rw_2",              pure_purify_watch_rw_2,              METH_VARARGS},
+    {"purify_watch_rw_4",              pure_purify_watch_rw_4,              METH_VARARGS},
+    {"purify_watch_rw_8",              pure_purify_watch_rw_8,              METH_VARARGS},
+    {"purify_watch_n",                 pure_purify_watch_n,                 METH_VARARGS},
+    {"purify_watch_info",              pure_purify_watch_info,              METH_VARARGS},
+    {"purify_watch_remove",            pure_purify_watch_remove,            METH_VARARGS},
+    {"purify_watch_remove_all",        pure_purify_watch_remove_all,        METH_VARARGS},
+    {"purify_describe",                pure_purify_describe,                METH_VARARGS},
+    {"purify_what_colors",             pure_purify_what_colors,             METH_VARARGS},
+    {"purify_is_running",              pure_purify_is_running,              METH_VARARGS},
+    {"purify_assert_is_readable",      pure_purify_assert_is_readable,      METH_VARARGS},
+    {"purify_assert_is_writable",      pure_purify_assert_is_writable,      METH_VARARGS},
 #if HAS_PURIFY_EXIT
     /* I wish I could include this, but I can't.  See the notes at the
      * top of the file.
      */
-    {"purify_exit",                    pure_purify_exit,                    1},
+    {"purify_exit",                    pure_purify_exit,                    METH_VARARGS},
 #endif /* HAS_PURIFY_EXIT */
 #endif /* PURIFY_H */
 #ifdef QUANTIFY_H
-    {"quantify_is_running",            pure_quantify_is_running,            1},
-    {"quantify_help",                  pure_quantify_help,                  1},
-    {"quantify_print_recording_state", pure_quantify_print_recording_state, 1},
-    {"quantify_start_recording_data",  pure_quantify_start_recording_data,  1},
-    {"quantify_stop_recording_data",   pure_quantify_stop_recording_data,   1},
-    {"quantify_is_recording_data",     pure_quantify_is_recording_data,  1},
+    {"quantify_is_running",            pure_quantify_is_running,            METH_VARARGS},
+    {"quantify_help",                  pure_quantify_help,                  METH_VARARGS},
+    {"quantify_print_recording_state", pure_quantify_print_recording_state, METH_VARARGS},
+    {"quantify_start_recording_data",  pure_quantify_start_recording_data,  METH_VARARGS},
+    {"quantify_stop_recording_data",   pure_quantify_stop_recording_data,   METH_VARARGS},
+    {"quantify_is_recording_data",     pure_quantify_is_recording_data,  METH_VARARGS},
     {"quantify_start_recording_system_calls",
-     pure_quantify_start_recording_system_calls, 1},
+     pure_quantify_start_recording_system_calls, METH_VARARGS},
     {"quantify_stop_recording_system_calls",
-     pure_quantify_stop_recording_system_calls, 1},
+     pure_quantify_stop_recording_system_calls, METH_VARARGS},
     {"quantify_is_recording_system_calls",
-     pure_quantify_is_recording_system_calls, 1},
+     pure_quantify_is_recording_system_calls, METH_VARARGS},
     {"quantify_start_recording_system_call",
-     pure_quantify_start_recording_system_call, 1},
+     pure_quantify_start_recording_system_call, METH_VARARGS},
     {"quantify_stop_recording_system_call",
-     pure_quantify_stop_recording_system_call, 1},
+     pure_quantify_stop_recording_system_call, METH_VARARGS},
     {"quantify_is_recording_system_call",
-     pure_quantify_is_recording_system_call, 1},
+     pure_quantify_is_recording_system_call, METH_VARARGS},
     {"quantify_start_recording_dynamic_library_data",
-     pure_quantify_start_recording_dynamic_library_data, 1},
+     pure_quantify_start_recording_dynamic_library_data, METH_VARARGS},
     {"quantify_stop_recording_dynamic_library_data",
-     pure_quantify_stop_recording_dynamic_library_data, 1},
+     pure_quantify_stop_recording_dynamic_library_data, METH_VARARGS},
     {"quantify_is_recording_dynamic_library_data",
-     pure_quantify_is_recording_dynamic_library_data, 1},
+     pure_quantify_is_recording_dynamic_library_data, METH_VARARGS},
     {"quantify_start_recording_register_window_traps",
-     pure_quantify_start_recording_register_window_traps, 1},
+     pure_quantify_start_recording_register_window_traps, METH_VARARGS},
     {"quantify_stop_recording_register_window_traps",
-     pure_quantify_stop_recording_register_window_traps, 1},
+     pure_quantify_stop_recording_register_window_traps, METH_VARARGS},
     {"quantify_is_recording_register_window_traps",
-     pure_quantify_is_recording_register_window_traps, 1},
+     pure_quantify_is_recording_register_window_traps, METH_VARARGS},
     {"quantify_disable_recording_data",
-     pure_quantify_disable_recording_data, 1},
-    {"quantify_clear_data",        pure_quantify_clear_data,        1},
-    {"quantify_save_data",         pure_quantify_save_data,         1},
-    {"quantify_save_data_to_file", pure_quantify_save_data_to_file, 1},
-    {"quantify_add_annotation",    pure_quantify_add_annotation,    1},
+     pure_quantify_disable_recording_data, METH_VARARGS},
+    {"quantify_clear_data",        pure_quantify_clear_data,        METH_VARARGS},
+    {"quantify_save_data",         pure_quantify_save_data,         METH_VARARGS},
+    {"quantify_save_data_to_file", pure_quantify_save_data_to_file, METH_VARARGS},
+    {"quantify_add_annotation",    pure_quantify_add_annotation,    METH_VARARGS},
 #endif /* QUANTIFY_H */
     {NULL,  NULL}			     /* sentinel */
 };
@@ -952,6 +952,8 @@ initpure()
 	PyObject *m, *d;
 
 	m = Py_InitModule("pure", pure_methods);
+	if (m == NULL)
+    		return;
 	d = PyModule_GetDict(m);
 
         /* this is bogus because we should be able to find this information
